"""
日志工具 - 统一的日志记录功能

提供统一的日志记录功能，支持不同级别的日志输出。
"""

import logging
import sys
from dataclasses import dataclass
from typing import Optional


@dataclass
class LoggerConfig:
    """日志配置"""

    level: str = "INFO"
    format: str = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    file_path: Optional[str] = None
    console_output: bool = True


def setup_logger(
    name: str = "abtree", config: Optional[LoggerConfig] = None
) -> logging.Logger:
    """
    设置日志记录器

    Args:
        name: 日志记录器名称
        config: 日志配置

    Returns:
        配置好的日志记录器
    """
    if config is None:
        config = LoggerConfig()

    # 创建日志记录器
    logger = logging.getLogger(name)
    logger.setLevel(getattr(logging, config.level.upper()))

    # 清除现有的处理器
    logger.handlers.clear()

    # 创建格式化器
    formatter = logging.Formatter(config.format)

    # 添加控制台处理器
    if config.console_output:
        console_handler = logging.StreamHandler(sys.stdout)
        console_handler.setFormatter(formatter)
        logger.addHandler(console_handler)

    # 添加文件处理器
    if config.file_path:
        file_handler = logging.FileHandler(config.file_path, encoding="utf-8")
        file_handler.setFormatter(formatter)
        logger.addHandler(file_handler)

    return logger


def get_logger(name: str = "abtree") -> logging.Logger:
    """
    获取日志记录器

    Args:
        name: 日志记录器名称

    Returns:
        日志记录器
    """
    return logging.getLogger(name)


# 预定义的日志记录器
_abtree_logger = None


def get_abtree_logger() -> logging.Logger:
    """
    获取 ABTree 主日志记录器

    Returns:
        ABTree 日志记录器
    """
    global _abtree_logger
    if _abtree_logger is None:
        _abtree_logger = setup_logger("abtree")
    return _abtree_logger


def log_tree_execution(tree_name: str, status: str, duration: float = 0.0) -> None:
    """
    记录行为树执行日志

    Args:
        tree_name: 行为树名称
        status: 执行状态
        duration: 执行时长
    """
    logger = get_abtree_logger()
    logger.info(
        f"行为树 '{tree_name}' 执行完成 - 状态: {status}, 时长: {duration:.3f}s"
    )


def log_node_execution(node_name: str, node_type: str, status: str) -> None:
    """
    记录节点执行日志

    Args:
        node_name: 节点名称
        node_type: 节点类型
        status: 执行状态
    """
    logger = get_abtree_logger()
    logger.debug(f"节点执行 - 名称: {node_name}, 类型: {node_type}, 状态: {status}")


def log_blackboard_access(key: str, operation: str, value: any = None) -> None:
    """
    记录黑板访问日志

    Args:
        key: 键名
        operation: 操作类型
        value: 值
    """
    logger = get_abtree_logger()
    if value is not None:
        logger.debug(f"黑板访问 - 键: {key}, 操作: {operation}, 值: {value}")
    else:
        logger.debug(f"黑板访问 - 键: {key}, 操作: {operation}")


def log_event(event_name: str, source: str = None, data: any = None) -> None:
    """
    记录事件日志

    Args:
        event_name: 事件名称
        source: 事件源
        data: 事件数据
    """
    logger = get_abtree_logger()
    if source and data:
        logger.info(f"事件 - 名称: {event_name}, 源: {source}, 数据: {data}")
    elif source:
        logger.info(f"事件 - 名称: {event_name}, 源: {source}")
    else:
        logger.info(f"事件 - 名称: {event_name}")


def log_error(message: str, error: Exception = None) -> None:
    """
    记录错误日志

    Args:
        message: 错误消息
        error: 异常对象
    """
    logger = get_abtree_logger()
    if error:
        logger.error(f"错误: {message}", exc_info=error)
    else:
        logger.error(f"错误: {message}")


def log_warning(message: str) -> None:
    """
    记录警告日志

    Args:
        message: 警告消息
    """
    logger = get_abtree_logger()
    logger.warning(f"警告: {message}")


def log_info(message: str) -> None:
    """
    记录信息日志

    Args:
        message: 信息消息
    """
    logger = get_abtree_logger()
    logger.info(message)


def log_debug(message: str) -> None:
    """
    记录调试日志

    Args:
        message: 调试消息
    """
    logger = get_abtree_logger()
    logger.debug(message)
